#@+leo-ver=4
#@+node:@file ../scripts/leo_interface.py
"""
This file implements an interface to XML generation,
so that the resulting file can be processed by leo.

It can be used to browse tree structured data in leo.

class file represents the whole leo file.
class leo_node has a headline and body text.

See the end of this file for a minimal example on
how to use these classes.
"""

#@@language python

from string import replace

#@+others
#@+node:escape
def escape(s):

	s = replace(s, '&', "&amp;")
	s = replace(s, '<', "&lt;")
	s = replace(s, '>', "&gt;")
	return s
#@-node:escape
#@+node:class node
error_count = 0

class node:

	"""Abstrace class for generating xml."""

	#@	@+others
	#@+node:__init__
	def __init__(self):
		self.children = []
	#@-node:__init__
	#@+node:gen
	def gen(self, file):
		pass
	#@-node:gen
	#@+node:add_child
	def add_child(self, child):
		self.children.append(child)
	#@-node:add_child
	#@+node:mark
	def mark(self, file, marker, func, newline=True):
		file.write("<%s>" % marker)
		if newline:
			file.write("\n")
		func(file)
		file.write("</%s>\n" % marker)
	#@-node:mark
	#@+node:mark_with_attributes
	def mark_with_attributes(self, file, marker, attribute_list, func, newline=True):
		write = file.write
		write("<")
		write(marker)
		write(" ")
		for name, value in attribute_list:
			write('%s="%s" ' % (name, value))
		write(">")
		if newline:
			write("\n")
		func(file)
		write("</%s>" % marker)
		if newline:
			write("\n")
	#@-node:mark_with_attributes
	#@+node:mark_with_attributes_short
	def mark_with_attributes_short(self, file, marker, attribute_list):
		write = file.write
		write("<")
		write(marker)
		write(" ")
		for name, value in attribute_list:
			write('%s="%s" ' % (name, value))
		write("/>\n")
	#@-node:mark_with_attributes_short
	#@+node:gen_children
	def gen_children(self, file):
		for child in self.children:
			child.gen(file)
	#@-node:gen_children
	#@-others
#@-node:class node
#@+node:class file
class file(node):

	"""Leo specific class representing a file."""

	#@	@+others
	#@+node:header
	def header(self, file):
		self.mark_with_attributes_short(file, "leo_header",
								  (("file_format", "1"),
								   ("tnodes", `self.nr_tnodes()`),
								   ("max_tnode_index", `self.max_tnode_index()`),
								   ("clone_windows", "0")))
	#@-node:header
	#@+node:sss
	def sss(self, file):
		file.write("sss")
	#@-node:sss
	#@+node:preferences
	def preferences(self, file):
		pass
	#@-node:preferences
	#@+node:empty
	def empty(self, file):
		pass
	#@-node:empty
	#@+node:find_panel_settings
	def find_panel_settings(self, file):
		self.mark(file, "find_string", self.empty, newline=False)
		self.mark(file, "change_string", self.empty, newline=False)
	#@-node:find_panel_settings
	#@+node:gen
	def gen(self, file):
		global error_count
		error_count = 0
		file.write('<?xml version="1.0" encoding="UTF-8"?>\n')
		self.mark(file, "leo_file", self.gen1)
	#@-node:gen
	#@+node:gen1
	def gen1(self, file):
		self.header(file)
	
		# This is a shortcut.
		file.write("""<globals body_outline_ratio="0.5">
	<global_window_position top="0" left="2" height="400" width="700"/>
	<global_log_window_position top="0" left="0" height="0" width="0"/>
	</globals>
		""")
		self.mark(file, "preferences", self.preferences)
		self.mark(file, "find_panel_settings", self.find_panel_settings)
		self.mark(file, "vnodes", self.gen_vnodes)
		self.mark(file, "tnodes", self.gen_tnodes)
	#@-node:gen1
	#@+node:gen_vnodes
	def gen_vnodes(self, file):
		for child in self.children:
			child.gen_vnodes(file)
	#@-node:gen_vnodes
	#@+node:gen_tnodes
	def gen_tnodes(self, file):
		for child in self.children:
			child.gen_tnodes(file)
	#@-node:gen_tnodes
	#@+node:nr_tnodes
	def nr_tnodes(self):
		return leo_node.count
	#@-node:nr_tnodes
	#@+node:max_tnode_index
	def max_tnode_index(self):
		return leo_node.count
	#@-node:max_tnode_index
	#@-others
#@-node:class file
#@+node:class leo_node
class leo_node(node):

	"""Leo specific class representing a node."""

	count = 0

	#@	@+others
	#@+node:__init__
	def __init__(self, headline='', body=''):
	
		node.__init__(self)
		leo_node.count += 1
		self.nr = leo_node.count
		self.headline =  headline
		self.body = body
	#@-node:__init__
	#@+node:set_headline
	def set_headline(self, headline):
	
		self.headline = headline
	#@-node:set_headline
	#@+node:set_body
	def set_body(self, body):
	
		self.body = body
	#@-node:set_body
	#@+node:get_headline
	def get_headline(self, headline):
	
		return self.headline
	#@-node:get_headline
	#@+node:get_body
	def get_body(self, body):
	
		return self.body
	#@-node:get_body
	#@+node:gen_vnodes
	def gen_vnodes(self, file):
	
		self.mark_with_attributes(file, "v", (
			("t", "T" + `self.nr`),
			), self.gen_vnodes1)
	#@-node:gen_vnodes
	#@+node:gen_vnodes1
	def gen_vnodes1(self, file):
	
		self.mark(file, "vh", self.write_headline_escaped, newline=False)
		for child in self.children:
			child.gen_vnodes(file)
	#@-node:gen_vnodes1
	#@+node:write_headline
	def write_headline(self, file):
	
		file.write(self.headline) 
	#@-node:write_headline
	#@+node:gen_tnodes
	def gen_tnodes(self, file):
	
		self.mark_with_attributes(file, "t", (
			("tx", "T" + `self.nr`),
			), self.gen_tnodes1, newline=False)
		for child in self.children:
			child.gen_tnodes(file)
	#@-node:gen_tnodes
	#@+node:gen_tnodes1
	def gen_tnodes1(self, file):
	
		self.write_body_escaped(file)
	#@-node:gen_tnodes1
	#@+node:write_body
	def write_body(self, file):
	
		file.write(self.body) 
	#@-node:write_body
	#@+node:write_headline_escaped
	def write_headline_escaped(self, file):
	
		"For now, do not escape"
		file.write(escape(self.headline))
	#@-node:write_headline_escaped
	#@+node:write_body_escaped
	def write_body_escaped(self, file):
	
		"For now, do not escape"
	
		escaped = escape(self.body)
		try:
			file.write(escaped)
		except UnicodeError:
			# Skip the characters which can not
			# be written out
			global error_count
			for ch in escaped:
				try:
					file.write(ch)
				except UnicodeError:
					error_count += 1
	#@-node:write_body_escaped
	#@-others
#@nonl
#@-node:class leo_node
#@+node:leotree
def leotree():

	f = file()
	return f
#@-node:leotree
#@-others

if __name__ == "__main__":
	import sys
	f = leotree()
	r = leo_node("Some headline", "some Body")
	f.add_child(r)
	f.gen(sys.stdout)
#@nonl
#@-node:@file ../scripts/leo_interface.py
#@-leo
